@@ -381,8 +381,17 @@ class RecurringComponent(Component):
                     try:
                         dtstart = self.dtstart.value
                     except AttributeError, KeyError:
-                        # if there's no dtstart, just return None
-                        return None
+                        # Special for VTODO - try DUE property instead
+                        try:
+                            if self.name == "VTODO":
+                                dtstart = self.due.value
+                            else:
+                                # if there's no dtstart, just return None
+                                return None
+                        except AttributeError, KeyError:
+                            # if there's no due, just return None
+                            return None
+
                     # rrulestr complains about unicode, so cast to str
                     rule = dateutil.rrule.rrulestr(str(line.value),
                                                    dtstart=dtstart)
@@ -426,7 +435,16 @@ class RecurringComponent(Component):
         return rruleset
 
     def setrruleset(self, rruleset):
-        dtstart = self.dtstart.value
+        
+        # Get DTSTART from component (or DUE if no DTSTART in a VTODO)
+        try:
+            dtstart = self.dtstart.value
+        except AttributeError, KeyError:
+            if self.name == "VTODO":
+                dtstart = self.due.value
+            else:
+                raise
+            
         isDate = datetime.date == type(dtstart)
         if isDate:
             dtstart = datetime.datetime(dtstart.year,dtstart.month, dtstart.day)
@@ -492,7 +510,7 @@ class RecurringComponent(Component):
 
                     if rule._bymonth is not None and len(rule._bymonth) > 0:
                         if (rule._byweekday is not None or
-                            len(rrule._bynweekday or ()) > 0 or
+                            len(rule._bynweekday or ()) > 0 or
                             not (rule._freq == dateutil.rrule.YEARLY and
                                  len(rule._bymonth) == 1 and
                                  rule._bymonth[0] == rule._dtstart.month)):
@@ -526,10 +544,44 @@ class RecurringComponent(Component):
         else:
             super(RecurringComponent, self).__setattr__(name, value)
 
-class RecurringBehavior(behavior.Behavior):
+class TextBehavior(behavior.Behavior):
+    """Provide backslash escape encoding/decoding for single valued properties.
+    
+    TextBehavior also deals with base64 encoding if the ENCODING parameter is
+    explicitly set to BASE64.
+    
+    """
+    base64string = 'BASE64' # vCard uses B
+    
+    @classmethod
+    def decode(cls, line):
+        """Remove backslash escaping from line.value."""
+        if line.encoded:
+            encoding = getattr(line, 'encoding_param', None)
+            if encoding and encoding.upper() == cls.base64string:
+                line.value = line.value.decode('base64')
+            else:
+                line.value = stringToTextValues(line.value)[0]
+            line.encoded=False
+    
+    @classmethod
+    def encode(cls, line):
+        """Backslash escape line.value."""
+        if not line.encoded:
+            encoding = getattr(line, 'encoding_param', None)
+            if encoding and encoding.upper() == cls.base64string:
+                line.value = line.value.encode('base64').replace('\n', '')
+            else:
+                line.value = backslashEscape(line.value)
+            line.encoded=True
+
+class VCalendarComponentBehavior(behavior.Behavior):
+    defaultBehavior = TextBehavior
+    isComponent = True
+
+class RecurringBehavior(VCalendarComponentBehavior):
     """Parent Behavior for components which should be RecurringComponents."""
     hasNative = True
-    isComponent = True
     
     @staticmethod
     def transformToNative(obj):
@@ -584,6 +636,8 @@ class DateTimeBehavior(behavior.Behavior):
         if obj.value.tzinfo is None:
             obj.params['X-VOBJ-FLOATINGTIME-ALLOWED'] = ['TRUE']
         if obj.params.get('TZID'):
+            # Keep a copy of the original TZID around
+            obj.params['X-VOBJ-ORIGINAL-TZID'] = obj.params['TZID']
             del obj.params['TZID']
         return obj
 
@@ -596,6 +650,10 @@ class DateTimeBehavior(behavior.Behavior):
             obj.value = dateTimeToString(obj.value, cls.forceUTC)
             if not cls.forceUTC and tzid is not None:
                 obj.tzid_param = tzid
+            if obj.params.get('X-VOBJ-ORIGINAL-TZID'):
+                if not hasattr(obj, 'tzid_param'):
+                    obj.tzid_param = obj.params['X-VOBJ-ORIGINAL-TZID']
+                del obj.params['X-VOBJ-ORIGINAL-TZID']
 
         return obj
 
@@ -614,9 +672,12 @@ class DateOrDateTimeBehavior(behavior.Behavior):
         obj.isNative = True
         if obj.value == '': return obj
         obj.value=str(obj.value)
-        obj.value=parseDtstart(obj)
+        obj.value=parseDtstart(obj, allowSignatureMismatch=True)
         if getattr(obj, 'value_param', 'DATE-TIME').upper() == 'DATE-TIME':
-            if hasattr(obj, 'tzid_param'): del obj.tzid_param
+            if hasattr(obj, 'tzid_param'):
+                # Keep a copy of the original TZID around
+                obj.params['X-VOBJ-ORIGINAL-TZID'] = obj.tzid_param
+                del obj.tzid_param
         return obj
 
     @staticmethod
@@ -688,37 +749,6 @@ class MultiDateBehavior(behavior.Behavior):
                 obj.value = ','.join(transformed)
             return obj
 
-class TextBehavior(behavior.Behavior):
-    """Provide backslash escape encoding/decoding for single valued properties.
-    
-    TextBehavior also deals with base64 encoding if the ENCODING parameter is
-    explicitly set to BASE64.
-    
-    """
-    base64string = 'BASE64' # vCard uses B
-    
-    @classmethod
-    def decode(cls, line):
-        """Remove backslash escaping from line.value."""
-        if line.encoded:
-            encoding = getattr(line, 'encoding_param', None)
-            if encoding and encoding.upper() == cls.base64string:
-                line.value = line.value.decode('base64')
-            else:
-                line.value = stringToTextValues(line.value)[0]
-            line.encoded=False
-    
-    @classmethod
-    def encode(cls, line):
-        """Backslash escape line.value."""
-        if not line.encoded:
-            encoding = getattr(line, 'encoding_param', None)
-            if encoding and encoding.upper() == cls.base64string:
-                line.value = line.value.encode('base64').replace('\n', '')
-            else:
-                line.value = backslashEscape(line.value)
-            line.encoded=True
-
 class MultiTextBehavior(behavior.Behavior):
     """Provide backslash escape encoding/decoding of each of several values.
     
@@ -742,12 +772,11 @@ class MultiTextBehavior(behavior.Behavior):
     
 
 #------------------------ Registered Behavior subclasses -----------------------
-class VCalendar2_0(behavior.Behavior):
+class VCalendar2_0(VCalendarComponentBehavior):
     """vCalendar 2.0 behavior."""
     name = 'VCALENDAR'
     description = 'vCalendar 2.0, also known as iCalendar.'
     versionString = '2.0'
-    isComponent = True
     sortFirst = ('version', 'calscale', 'method', 'prodid', 'vtimezone')
     knownChildren = {'CALSCALE':  (0, 1, None),#min, max, behaviorRegistry id
                      'METHOD':    (0, 1, None),
@@ -805,11 +834,10 @@ class VCalendar2_0(behavior.Behavior):
                 obj.add(TimezoneComponent(tzinfo=getTzid(tzid)))
 registerBehavior(VCalendar2_0)
 
-class VTimezone(behavior.Behavior):
+class VTimezone(VCalendarComponentBehavior):
     """Timezone behavior."""
     name = 'VTIMEZONE'
     hasNative = True
-    isComponent = True
     description = 'A grouping of component properties that defines a time zone.'
     sortFirst = ('tzid', 'last-modified', 'tzurl', 'standard', 'daylight')
     knownChildren = {'TZID':         (1, 1, None),#min, max, behaviorRegistry id
@@ -851,10 +879,10 @@ class VTimezone(behavior.Behavior):
         
 registerBehavior(VTimezone)
 
-class DaylightOrStandard(behavior.Behavior):
+class DaylightOrStandard(VCalendarComponentBehavior):
     hasNative = False
-    isComponent = True
-    knownChildren = {'DTSTART':      (1, 1, None)}#min, max, behaviorRegistry id
+    knownChildren = {'DTSTART':      (1, 1, None),#min, max, behaviorRegistry id
+                     'RRULE':        (0, 1, None)}
 
 registerBehavior(DaylightOrStandard, 'STANDARD')
 registerBehavior(DaylightOrStandard, 'DAYLIGHT')
@@ -1002,7 +1030,7 @@ class VJournal(RecurringBehavior):
 registerBehavior(VJournal)
 
 
-class VFreeBusy(behavior.Behavior):
+class VFreeBusy(VCalendarComponentBehavior):
     """Free/busy state behavior.
 
     >>> vfb = newFromBehavior('VFREEBUSY')
@@ -1020,7 +1048,6 @@ class VFreeBusy(behavior.Behavior):
 
     """
     name='VFREEBUSY'
-    isComponent = True
     description='A grouping of component properties that describe either a \
                  request for free/busy time, describe a response to a request \
                  for free/busy time or describe a published set of busy time.'
@@ -1041,10 +1068,9 @@ class VFreeBusy(behavior.Behavior):
 registerBehavior(VFreeBusy)
 
 
-class VAlarm(behavior.Behavior):
+class VAlarm(VCalendarComponentBehavior):
     """Alarm behavior."""
     name='VALARM'
-    isComponent = True
     description='Alarms describe when and how to provide alerts about events \
                  and to-dos.'
     knownChildren = {'ACTION':       (1, 1, None),#min, max, behaviorRegistry id
@@ -1286,6 +1312,14 @@ class FreeBusy(PeriodBehavior):
     forceUTC = True
 registerBehavior(FreeBusy)
 
+class RRule(behavior.Behavior):
+    """
+    Dummy behavior to avoid having RRULEs being treated as text lines (and thus
+    having semi-colons inaccurately escaped).
+    """
+registerBehavior(RRule, 'RRULE')
+registerBehavior(RRule, 'EXRULE')
+
 #------------------------ Registration of common classes -----------------------
 
 utcDateTimeList = ['LAST-MODIFIED', 'CREATED', 'COMPLETED', 'DTSTAMP']
@@ -1399,8 +1433,7 @@ def isDuration(s):
     s = string.upper(s)
     return (string.find(s, "P") != -1) and (string.find(s, "P") < 2)
 
-def stringToDate(s, tzinfos=None):
-    if tzinfos != None: print "Didn't expect a tzinfos here"
+def stringToDate(s):
     year  = int( s[0:4] )
     month = int( s[4:6] )
     day   = int( s[6:8] )
@@ -1603,13 +1636,26 @@ def stringToDurations(s, strict=False):
             state = "error"
             error("error: unknown state: '%s' reached in %s" % (state, s))
 
-def parseDtstart(contentline):
+def parseDtstart(contentline, allowSignatureMismatch=False):
+    """Convert a contentline's value into a date or date-time.
+    
+    A variety of clients don't serialize dates with the appropriate VALUE
+    parameter, so rather than failing on these (technically invalid) lines,
+    if allowSignatureMismatch is True, try to parse both varieties.
+    
+    """
     tzinfo = getTzid(getattr(contentline, 'tzid_param', None))
     valueParam = getattr(contentline, 'value_param', 'DATE-TIME').upper()
     if valueParam == "DATE":
         return stringToDate(contentline.value)
     elif valueParam == "DATE-TIME":
-        return stringToDateTime(contentline.value, tzinfo)
+        try:
+            return stringToDateTime(contentline.value, tzinfo)
+        except:
+            if allowSignatureMismatch:
+                return stringToDate(contentline.value)
+            else:
+                raise
 
 def stringToPeriod(s, tzinfo=None):
     values   = string.split(s, "/")
